"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""
from models.multicluster import MultiCluster
import threading
session_locks = {}  # Synchronize concurrent API calls received for a session


def get_info(session, session_manager, cluster_manager, xdr_port):
    """
    Request multicluster data from multicluster object.
    :param session: Expected Format-> {'session': <session-UUID>}
    :param session_manager: session_manager obj from app
    :param cluster_manager: cluster_manager obj from app
    :param xdr_port: XDR port of AS
    :return: {
        'status': <success/failure>,
        <data/error>: <mc-data/error-msg>
    }
    """
    response = {'status': 'failure', 'error': 'No session found'}
    session_key = session.get('session', None)  # Get current session
    if not session_key:
        return response

    global session_locks
    if session_key in session_locks:
        lock = session_locks[session_key]
    else:
        session_locks[session_key] = lock = threading.RLock()

    lock.acquire()
    try:
        # Get all clusters from session
        clusters = session_manager.get_clusters_from_session(session_key)
        if not clusters:
            return dict(status='failure', error='No cluster object found')

        cluster_id = clusters.pop()['cluster_id']  # Get one cluster object
        cluster_obj = cluster_manager.get_cluster_by_id(cluster_id)

        try:
            xdr_port = int(xdr_port)  # Check xdr_port type
        except ValueError:
            xdr_port = 3004  # Set to default port

        # Get multicluster object from session
        multicluster_obj = session_manager.get_multicluster(session_key)

        if not multicluster_obj:  # Create new multicluster object
            cluster_name = session_manager.get_cluster_name(
                session_key, cluster_id)
            multicluster_obj = MultiCluster(cluster_obj, cluster_name, xdr_port)
            # Add multicluster object to session
            session_manager.add_multicluster(session_key, multicluster_obj)

        # Get all explored clusters in multicluster obj
        # Compare it with cluster objects in session_manager
        # And add cluster objects that are not already present in mc
        mc_discovered_objects = multicluster_obj.get_discovered_cluster_objects(
        ).values()

        while True:
            if cluster_obj not in mc_discovered_objects:
                cluster_name = session_manager.get_cluster_name(
                    session_key, cluster_id)
                multicluster_obj.add_cluster_object(
                    cluster_obj, cluster_name, xdr_port)

            if not clusters:
                break

            cluster_id = clusters.pop()['cluster_id']
            cluster_obj = cluster_manager.get_cluster_by_id(cluster_id)

        response = {
            'status': 'success',
            'data': multicluster_obj.get_multicluster_data(
                xdr_port, session, session_manager, cluster_manager)
        }
    finally:
        lock.release()
    return response
